


local origUnit = Unit

Unit = Class(origUnit) {

    OnPaused = function(self)
        if (self.unitCanBeDisabled) then
            self.unitIsPaused = true
            self:SetConsumptionPerSecondEnergy( 0 )
            self:SetEnergyMaintenanceConsumptionOverride( 0 )
        end
        origUnit.OnPaused(self)
    end,

    OnUnpaused = function(self)
        if (self.unitCanBeDisabled) then
            self.unitIsPaused = false
            self:SetConsumptionPerSecondEnergy( 0.1  *  self.minconsumption )
            self:SetEnergyMaintenanceConsumptionOverride( 0.1  *  self.minconsumption )
        end
        origUnit.OnUnpaused(self)
    end,



    OnStopBeingBuilt = function(self, builder, layer)
      self.unitIsDisabled = false
      self.unitIsPaused = false

      origUnit.OnStopBeingBuilt (self, builder, layer)


      local bp = self:GetBlueprint()
      local bpEcon = bp.Economy

      self.unitCanBeDisabled = false
      if (bp.General.CanDisableUnit ) then
           self.unitCanBeDisabled = bp.General.CanDisableUnit 
      end

      self.minconsumption = bpEcon.MinConsumptionPerSecondEnergy
      self.consumption = bpEcon.MaintenanceConsumptionPerSecondEnergy

      # add support for a blueprint toggle that turns consumption on.
      if (bpEcon.initialMaintenance) then

        self:SetMaintenanceConsumptionActive ()
        self:UpdateConsumptionValues()

        self:ForkThread(self.EnergyWatchThread )
        #note that Unit:ForkThread does both call ForkThread and adds self to the trash

      end

    end,




    #Watch the economy
    EnergyWatchThread = function(self)
        local fraction = 1
        local speedfraction = 1
        local forever = true
        local fractionPrev = fraction

        #minimum reduction so that units don't stop completely.
        local minimumspeedfraction = 0.2

        # to stabilize reactivation of a unit
        # it will only become active if there is enough energy to support a fairly active unit.
        local threshold = 1.0


        while forever do
            WaitSeconds(1.0)

                    #local army = self:GetArmy()   #this is just a number, 1 to ..
                    #local brain = GetArmyBrain(army)
                    #local energyfractionstored = brain:GetArmyStat("Economy_Ratio_Energy", 0.0).Value
                    ##this shows how much energy there is in store for your army 0=empty, 1=full
                    ##although it is not always exactly 0, it can be like 1E-10.
                    #local energyover = energyfractionstored > 1E-6

            #### active unit

            if (not self.unitIsDisabled) and (not self.unitIsPaused) then
 
 
                #self:SetCustomName("keep going army= ".. army .. " E= " .. energyfractionstored )
                #self:SetCustomName("keep going")

                if self.MaintenanceConsumption then
 
                    local fractioncurrent = self:GetResourceConsumed()
                    if ( fractioncurrent < 1 ) then
                        #slowly reduce unit efficiency
                        fraction = 0.9 * fraction
                    else
                        #slowly increase unit efficiency
                        fraction = 0.9 * fraction + 0.1
 
                        if (fraction > 0.99) then
                            #this should not be possible but well...
                            fraction = 1
                        end
                    end


                    

                else
                    fraction = 1.0
                end

                speedfraction = fraction
                if speedfraction < minimumspeedfraction then
                    speedfraction = minimumspeedfraction
                end

                # disable (stun) a unit
                if (fraction < 0.1) then
                    #self:SetCustomName("order stop")
                    self.unitIsDisabled = true
                    self:SetEnergyMaintenanceConsumptionOverride( 0 )
                    self:SetConsumptionPerSecondEnergy( 0 )
                else

                    #you should not make the weapon rate too small, cause it looks like it's only updated after fire.
                    #so if it takes long to fire, it will take long to use the new value.
                    local weaponfraction = fraction
                    if (weaponfraction < 0.1) then
                         weaponfraction = 0.1
                    end


                    for i = 1, self:GetWeaponCount() do
                         local wep = self:GetWeapon(i)
                         local bp = wep:GetBlueprint()
                         if (bp and bp.RateOfFire) then
                             wep:ChangeRateOfFire(weaponfraction * bp.RateOfFire)
                         end
                     end

                    self:SetSpeedMult(speedfraction)
                    self:SetAccMult(speedfraction)
                    self:SetTurnMult(speedfraction)


                    # the energy used to run the thing, which is always happening
                    local energybase =  fraction * self.minconsumption 

                    # energy for extra features like cloak etc.
                    # these are toggled on/off, through self.MaintenanceConsumption
                    local energymaintenance = 0
                    if self.MaintenanceConsumption then
                        energymaintenance = self.consumption - self.minconsumption
                    end

                    #energy for building a unit... toggled on/off by self.ActiveConsumption.
                    # this is needed for the ACUs cause those can build.
                    local energyactive = 0
                    if self.ActiveConsumption then
                        energyactive = self:GetActiveEnergyConsumption()
                    end

                    self:SetConsumptionPerSecondEnergy( energybase + energymaintenance + energyactive  )
                    self:SetEnergyMaintenanceConsumptionOverride( energybase )

                    #self:UpdateConsumptionValues()
                    #you are not allowed to call this one, cause it will override with the blueprint minimum consumption setting

                end

            end


            #note that the value of self.unitIsDisabled can be changed in the previous if-block
            #so we repeat the check here
            if (self.unitIsDisabled) or (self.unitIsPaused) then

                #### inactive unit

                if (not self.unitIsPaused) then

                    #add an extra delay so that values (of other units) propagate into the economy.
                    #this delay can be fairly long cause when you run out of energy, it takes a while to replenish anyway
                    #add random so that not all units update simultaneously.
                    local extradelay = 1.0 + Random(3.0)

                    #add (plenty of) extra stun first so that units wont start moving during the wait time.
                    self:SetStunned(extradelay * 2 + 2.0)

                    WaitSeconds(extradelay)


                    # test if it should be enabled again automatically. Temporarily set the energy consumption value


# too bad the following does not work properly
# cause it looks like self:GetResourceConsumed() looks back one iteration
# and doesn't give an instant update.
# or maybe the SetConsumptionPerSecondEnergy(x) function takes an iteration to propagate
# who knows.
#                    self:SetConsumptionPerSecondEnergy(900000)# threshold  *  self.minconsumption )
#                    local fractioncurrent = self:GetResourceConsumed()
#                    self:SetConsumptionPerSecondEnergy( 0 )
#
#so instead I will need to access an army stat
#

                    local army = self:GetArmy()   #this is just a number, 1 to ..
                    local brain = GetArmyBrain(army)
                    local energyfractionstored = brain:GetArmyStat("Economy_Ratio_Energy", 0.0).Value
                    #this shows how much energy there is in store for your army 0=empty, 1=full
                    #although it is not always exactly 0, it can be like 1E-10.
                    local energyover = energyfractionstored > 1E-6


                    if (energyover) then
                    #    self:SetCustomName("restart")

                        self.unitIsDisabled = false
                        # enable the unit again if the economy has a surplus so that it can maintain the unit
                        fraction = 0.1
                        speedfraction = fraction 

                        self:SetConsumptionPerSecondEnergy( fraction *  self.minconsumption )

                        self:SetSpeedMult(speedfraction)
                        self:SetAccMult(speedfraction)
                        self:SetTurnMult(speedfraction)
                    end
                end


                if (self.unitIsPaused) or (self.unitIsDisabled) then
                    #self:SetCustomName("stay still")
                    #if the unit is paused or stunned, then all comes to complete stop (both eco and speed).
                    fraction = 0
                    speedfraction = 0

                    #add plenty of stuntime
                    self:SetStunned(2.0)

                    self:SetConsumptionPerSecondEnergy( 0 )
                end
            end




                fractionPrev = fraction

        end
    end,


#copied from Units.lua, function UpdateConsumptionValues()

    GetActiveEnergyConsumption = function(self)

        local energy_rate = 0

        if self.ActiveConsumption then

            local focus = self:GetFocusUnit()
            local time = 1
            local mass = 0
            local energy = 0
            if self.WorkItem then
                time, energy, mass = Game.GetConstructEconomyModel(self, self.WorkItem)
            elseif focus then
                # bonuses are already factored in by GetBuildCosts
                time, energy, mass = self:GetBuildCosts(focus:GetBlueprint())
            end
            energy = energy * (self.EnergyBuildAdjMod or 1)
            if energy < 1 then
                energy = 1
            end
            time = time * (self.BuildTimeMultiplier or 1)
            if (time) then
                energy_rate = energy / time
            end
        end

        return energy_rate
    end,



#test if this is called to override values despite our suppression.
    UpdateConsumptionValues = function(self)
        if (self.unitIsPaused) or (self.unitIsDisabled) then
            return
        else
            origUnit.UpdateConsumptionValues(self)
        end
    end,


}






